<?php

namespace Drupal\entity_reference_field_create_link\Plugin\Field\FieldWidget;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\Plugin\Field\FieldWidget\EntityReferenceAutocompleteWidget;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'entity_reference_autocomplete' widget.
 *
 * @FieldWidget(
 *   id = "entity_reference_field_create_link",
 *   label = @Translation("Autocomplete (with create link)"),
 *   description = @Translation("An autocomplete text field with a link to
 *   create a new entity."),
 *   field_types = {
 *     "entity_reference"
 *   }
 * )
 */
class EntityReferenceAutocompleteCreateLink extends EntityReferenceAutocompleteWidget {

  /**
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   * @param array $configuration
   * @param string $plugin_id
   * @param array $plugin_definition
   *
   * @return \Drupal\Core\Field\WidgetBase|\Drupal\entity_reference_field_create_link\Plugin\Field\FieldWidget\EntityReferenceAutocompleteCreateLink
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);

    $instance->entityTypeManager = $container->get('entity_type.manager');

    return $instance;
  }

  /**
   * @param string $routeName
   * @param string $routeParameterName
   * @param \Drupal\Core\Entity\EntityInterface $bundleObject
   *
   * @return array
   */
  private function getAddLink(string $routeName, string $routeParameterName, EntityInterface $bundleObject): array {
    return Link::createFromRoute(
      $this->t('Create @name', ['@name' => $bundleObject->label()]),
      $routeName,
      [$routeParameterName => $bundleObject->id()],
      [
        'attributes' => [
          'class' => ['button', 'button--action', 'button--small'],
          'target' => '_blank',
        ]
      ]
    )->toRenderable();
  }

  /**
   * @param \Drupal\Core\Field\FieldItemListInterface $items
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   * @param null $get_delta
   *
   * @return array
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @todo Support more entity types
   * @todo Support entity types without bundles
   */
  function form(FieldItemListInterface $items, array &$form, FormStateInterface $form_state, $get_delta = NULL): array {
    $build = parent::form($items, $form, $form_state, $get_delta);
    $build['add_links'] = [];

    foreach ($this->getSelectionHandlerSetting('target_bundles') as $bundle) {
      // There is no universal way to get the add route for a given entity type.
      $entityTypeId = $this->getFieldSetting('target_type');
      $linkKey = implode('__', [$entityTypeId, $bundle]);

      switch ($entityTypeId) {
        case 'node':
          $build['add_links'][$linkKey] = $this->getAddLink(
            'node.add',
            'node_type',
            $this->entityTypeManager
              ->getStorage('node_type')
              ->load($bundle)
          );
          break;
        case 'taxonomy_term':
          $build['add_links'][$linkKey] = $this->getAddLink(
            'entity.taxonomy_term.add_form',
            'taxonomy_vocabulary',
            $this->entityTypeManager
              ->getStorage('taxonomy_vocabulary')
              ->load($bundle)
          );
          break;
        case 'media':
          $build['add_links'][$linkKey] = $this->getAddLink(
            'entity.media.add_form',
            'media_type',
            $this->entityTypeManager
              ->getStorage('media_type')
              ->load($bundle)
          );
          break;
      }
    }

    return $build;
  }

}
